/********************************** (C) COPYRIGHT *******************************
 * File Name          : app_tmos.C
 * Author             : WCH
 * Version            : V1.0
 * Date               : 2023/8/5
 * Description        : 
 *******************************************************************************/

/*********************************************************************
 * INCLUDES
 */
#include "Touch.h"
#include "CONFIG.h"
#include "app_tmos.h"
#include "peripheral.h"
/*********************
 *      DEFINES
 *********************/
#define SLEEP_TRIGGER_TIME MS1_TO_SYSTEM_TIME(500) // 500ms
#define TRIGGER_TIME MS1_TO_SYSTEM_TIME(100)       // 100ms
#define WAKEUP_TIME MS1_TO_SYSTEM_TIME(5)          // 50ms

/**********************
 *      VARIABLES
 **********************/
tmosTaskID TouchKey_TaskID = 0x00;
uint16_t triggerTime = SLEEP_TRIGGER_TIME;

/**********************
 *  STATIC PROTOTYPES
 **********************/
static void TKY_PeripheralInit(void);
static void peripherals_EnterSleep(void);
static void peripherals_WakeUp(void);

/**********************
 *   GLOBAL FUNCTIONS
 **********************/

/*********************************************************************
 * @fn      tky_on_TMOS_dataProcess
 *
 * @brief   ݴTMOSӳɹ󽫻ȡֵ֪ͨʽϱλ
 *
 * @return  none
 */
void tky_on_TMOS_dataProcess(void)
{
    uint8_t key_val = 0;
    key_val = touch_GetKey();
    if (key_val != 0x00)
    {
        if (bleConnectState )
        {
            peripheralChar2Notify( &key_val, 1 );//ֵϱλ
        }
    }
}


/*********************************************************************
 * @fn      PeriodicDealData
 *
 * @brief    ״̬
 *
 * @return  none
 */
void PeriodicDealData(void)
{
    TKY_LoadAndRun(); //---ǰĲ---
//    GPIOTK_PinSleep(  );

    //---̬ʱлʾݡ߻ֵÿдʱ10wakeupʱ䣬˶ʱʱΪ5s---
    if (wakeUpCount)
    {
        wakeUpCount--;
//        dg_log("wakeUpCount: :%d\n", wakeUpCount);
        //---wakeUpCountΪ0̬ת---
        if (wakeUpCount == 0)
        {
        	touch_ScanEnterSleep();

            tmos_stop_task(TouchKey_TaskID, WAKEUP_DATA_DEAL_EVT);
            triggerTime = SLEEP_TRIGGER_TIME;
            /*-------------------------
             * Call your peripherals sleep function
             * -----------------------*/
            peripherals_EnterSleep();
        }
    }
    else //---״̬ʱɨ---
    {
        dg_log("wake up...\n");

        scanData = TKY_ScanForWakeUp(tkyQueueAll); //---ѡĶͨɨ---

        if (scanData) //---ɨ쳣ʽɨ躯ģʽ3~4---
        {
            TKY_SetSleepStatusValue(~scanData); //---״̬쳣״̬ͨΪ̬---
            for (uint8_t i = 0; i < 40; i++) //---һҪɨ64Σ20ϽԿɣеɨа£˳ѭɨ---
            {
                keyData = TKY_PollForFilter();
                if (keyData) //---һ⵽а£˳ѭɨ---
                {
                	touch_ScanWakeUp();

                    triggerTime = TRIGGER_TIME;
                    tky_DealData_start();
                    tmos_start_task(TouchKey_TaskID, WAKEUP_DATA_DEAL_EVT, 0);
                    /*-------------------------
                     * Call your peripherals WakeUp function
                     * -----------------------*/
                    peripherals_WakeUp();
                    break;
                }
            }
            if (keyData == 0)
            {
                TKY_SaveAndStop(); //---ؼĴб---
            }
        }
        else
        {
            TKY_SaveAndStop(); //---ؼĴб---
        }
    }
    TKY_SaveAndStop(); //---ؼĴб---
}


/*********************************************************************
 * @fn      tky_DealData_start
 *
 * @brief   ɨ迪
 *
 * @return  none
 */
void tky_DealData_start(void)
{
    tmos_set_event(TouchKey_TaskID, DEALDATA_EVT);
}

/*********************************************************************
 * @fn      tky_DealData_stop
 *
 * @brief   ɨֹͣ
 *
 * @return  none
 */
void tky_DealData_stop(void)
{
    tmos_stop_task(TouchKey_TaskID, DEALDATA_EVT);
}


/*********************************************************************
 * @fn      Touch_Key_ProcessEvent
 *
 * @brief   
 *
 * @return  none
 */
tmosEvents Touch_Key_ProcessEvent(tmosTaskID task_id, tmosEvents events)
{
    uint16_t res;

    if (events & WAKEUP_DATA_DEAL_EVT)
    {
    	touch_KeyScan();
        tky_on_TMOS_dataProcess();
#if TKY_SLEEP_EN
        if (wakeupflag)
#endif
            tmos_start_task(TouchKey_TaskID, WAKEUP_DATA_DEAL_EVT, WAKEUP_TIME);
        return (events ^ WAKEUP_DATA_DEAL_EVT);
    }

    if (events & DEALDATA_EVT)
    {
        PeriodicDealData();
#if TKY_SLEEP_EN
        if (!advState || wakeupflag)
#endif
            tmos_start_task(TouchKey_TaskID, DEALDATA_EVT, triggerTime);
        return (events ^ DEALDATA_EVT);
    }

#if PRINT_EN
    if (events & DEBUG_PRINT_EVENT)
    {
        touch_InfoDebug();

        tmos_start_task(TouchKey_TaskID, DEBUG_PRINT_EVENT, SLEEP_TRIGGER_TIME);
        return (events ^ DEBUG_PRINT_EVENT);
    }
#endif

    return 0;
}


/*********************************************************************
 * @fn      touch_on_TMOS_init
 *
 * @brief   ʼTMOS
 *
 * @return  none
 */
void touch_on_TMOS_init(void)
{
    TouchKey_TaskID = TMOS_ProcessEventRegister(Touch_Key_ProcessEvent);
    TKY_PeripheralInit();       /* ʼ裬米ͷ */
    touch_InitKey();

    wakeUpCount = 50; //---ʱ---
    wakeupflag = 1;   // óɻ״̬
    triggerTime = TRIGGER_TIME;
    TKY_SetSleepStatusValue(~tkyQueueAll);
#if TKY_SLEEP_EN
    tky_DealData_start();
#else
    tky_DealData_stop();
#endif

#if PRINT_EN
    tmos_set_event(TouchKey_TaskID, DEBUG_PRINT_EVENT);
#endif

    tmos_set_event(TouchKey_TaskID, WAKEUP_DATA_DEAL_EVT);
    dg_log("Touch Key init Finish!\n");
}


/**********************
 *   STATIC FUNCTIONS
 **********************/


/*********************************************************************
 * @fn      TKY_PeripheralInit
 *
 * @brief   ʼڳʼ봥ص蹦
 *
 * @return  none
 */
static void TKY_PeripheralInit(void)
{
    /*You code here*/
}

/*********************************************************************
 * @fn      peripherals_EnterSleep
 *
 * @brief   ˯ߺڴ׼ʱ
 *
 * @return  none
 */
static void peripherals_EnterSleep(void)
{
    /*You code here*/
}


/*********************************************************************
 * @fn      peripherals_WakeUp
 *
 * @brief   軽Ѻڴʱ
 *
 * @return  none
 */
static void peripherals_WakeUp(void)
{
    /*You code here*/
}
